@@ -47,6 +47,7 @@ OWN_OPTION_SPEC = dict( { 'caption': str,
     'convert': str,
     'show_source': str,
     'hidden': str,
+    #'background': str,
     })
 
 class PlotError(SphinxError):
@@ -63,24 +64,30 @@ class PlotDirective(directives.images.Figure):
   
     def run(self):
         self.arguments = ['']
-        total_options = self.options.copy()
 
+        #print("content: %s" %(self.content))
         tmp = ""
         for line in self.content:
             #Find the fist verb not starting with '#'
             if (line and (not line.lstrip().startswith('#'))):
                 tmp = line.split()[0]
+                if not self.options.get("caption", None):
+                    #If there is no :caption:, take the 1st line as caption.
+                    tmp_line = line.rstrip()
+                    if tmp_line[-1] == "\\":
+                        self.options["caption"] = tmp_line[:-1] + "..."
+                    else:
+                        self.options["caption"] = tmp_line
                 break
-        if (tmp == "convert" or tmp == "montage"):
+        if (tmp == "convert" or tmp == "magick" or tmp == "montage"):
             #in case convert and montage: we put the cmd into the text itself
             cmd = tmp
-            if (len(self.content) > 2) and (cmd in self.content[2]):
-                text = '\n'.join(self.content[2:])
-            else:
-                text = '\n'.join(self.content)
+            text = '\n'.join(self.content)
         else:
             cmd = self.content[0]
             text = '\n'.join(self.content[1:])
+
+        total_options = self.options.copy()
         own_options = dict([(k,v) for k,v in self.options.items() 
                                   if k in OWN_OPTION_SPEC])
 
@@ -186,7 +193,7 @@ def plot_image (app, plot):
     infullfn = path.join(app.builder.outdir, app.builder.imagedir, infname)
     ensuredir(path.join(app.builder.outdir, app.builder.imagedir))
     currpath = os.getcwd() # Record the current dir and return here afterwards
-    plot_format = options["plot_format"] # plot_format option must be set before.
+    plot_format = options.get("plot_format", None) # plot_format option must be set before.
     outfname = '%s-%s.%s' %(args[0], hashkey, plot_format)
     out = dict(outrelfn = posixpath.join(rel_imgpath, outfname),
         outfullfn = path.join(app.builder.outdir, app.builder.imagedir, outfname),
@@ -237,9 +244,10 @@ def plot_image (app, plot):
         lines.insert(0, 'set terminal %s size %s\n' %(terminal, size))
         text = ''.join(lines)
         args.append(infullfn)
-    elif args[0] == "convert" or args[0] == "montage":
-        #print("debug, args: %s" %(args))
+    elif args[0] == "convert" or args[0] == "magick" or args[0] == "montage":
+        #magick is the same as convert
         tmp = []
+        #print("debug, args: %s" %(args))
         if text:
             # The body is a shell script, meanwhile replace the last para as
             # output filename
@@ -262,9 +270,9 @@ def plot_image (app, plot):
             text = " ".join(args)
             #args = ["sh", infullfn]
         #print("debug, args: %s" %(args))
-    elif args[0] == "magick":
-        args = ["magick", "-script", infullfn]
-        text += "\n-write %s" %(out["outfullfn"])
+    #elif args[0] == "magick":
+    #    args = ["magick", "-script", infullfn]
+    #    text += "\n-write %s" %(out["outfullfn"])
     else:
         args.append(infullfn)
 
@@ -283,8 +291,8 @@ def plot_image (app, plot):
         os.chdir(currpath)
         raise PlotError('[1;31m%s[0m' %(err))
 
-    # We'd like to change something after image is generated:
     if options.get("convert", None):
+        # We'd like to change something after image is generated:
         c = "convert %s" %(out["outfullfn"])
         for i in StringIO(options["convert"]).readlines():
             if (i.lstrip()[0] != "#"):
@@ -293,7 +301,9 @@ def plot_image (app, plot):
         print(c)
         os.system(c)
 
-    if cmd == "convert" or cmd == "montage":
+    #print("arg[0]: %s, plot_format: %s, format: %s"
+    #        %(args[0], plot_format, app.builder.format))
+    if cmd == "convert" or cmd == "magick" or cmd == "montage":
         #print("text: %s" %(text))
         #Get the last word as the original output name
         for i in reversed(StringIO(text).readlines()):
@@ -301,17 +311,28 @@ def plot_image (app, plot):
                 conver_outfile = i.split()[-1]
                 break
         if (app.builder.format == "latex") and conver_outfile.split(".")[-1] == "gif":
-            print("montage %s -coalesce %s" %(conver_outfile, out["outfullfn"]))
-            os.system("montage %s -coalesce %s" %(conver_outfile, out["outfullfn"]))
+            print("montage %s -coalesce -tile 8 %s" %(conver_outfile, out["outfullfn"]))
+            os.system("montage %s -coalesce -tile 8 %s"
+                    %(conver_outfile, out["outfullfn"]))
             os.system("rm %s" %(conver_outfile))
         else:
             print("mv %s %s" %(conver_outfile, out["outfullfn"]))
             os.system("mv %s %s" %(conver_outfile, out["outfullfn"]))
-    elif (args[0] == "ditaa" or args[0] == "seqdiag") and\
-            (plot_format in ['pdf']):
+        #if options.get("background", None):
+        #    # Add a background
+        #    print("convert %s -background %s -flatten %s"
+        #            %(out["outfullfn"], options["background"], out["outfullfn"]))
+        #    os.system("convert %s -background %s -flatten %s"
+        #            %(out["outfullfn"], options["background"], out["outfullfn"]))
+    elif (args[0] == "ditaa" or args[0] == "dot" or args[0] == "seqdiag") \
+            and (plot_format in ['pdf']):
         # In fact ditaa don't support pdf, we convert the .svg to .pdf inkscape
-        print("mv %s %s-%s.svg" %(outfname, args[0], hashkey))
-        os.system("mv %s %s-%s.svg" %(outfname, args[0], hashkey))
+        if (args[0] == "dot"):
+            print("mv %s %s-%s.svg" %(out["outfullfn"], args[0], hashkey))
+            os.system("mv %s %s-%s.svg" %(out["outfullfn"], args[0], hashkey))
+        else:
+            print("mv %s %s-%s.svg" %(outfname, args[0], hashkey))
+            os.system("mv %s %s-%s.svg" %(outfname, args[0], hashkey))
 
         inkscape = os.system("which inkscape 2> /dev/null")
         if inkscape != 0:
@@ -343,7 +364,8 @@ def cmd_2_image (app, plot):
     options = plot['total_options']
 
     # Guess the suffix
-    if "convert" in args or "montage" in args or "magick" in args:
+    plot_format = "png"
+    if "convert" in args or "magick" in args or "montage" in args:
         if app.builder.format in ["html"]:
             # We get the suffix of the last word not in the comments
             for i in reversed(StringIO(text).readlines()):
@@ -355,14 +377,20 @@ def cmd_2_image (app, plot):
             plot_format = "png"
     elif "dot" in args:
         #dot
-        if "-T" in args:
-            # Guess the plot_format if -TXXX is given
-            for param in args.split():
-                if "-T" in param:
-                    plot_format = param[2:]
-        else:
+        found = False
+        # Guess the plot_format if -TXXX is given
+        for param in args:
+            if "-T" in param:
+                plot_format = param[2:]
+                found = True
+                break
+        if (not found):
             # Set the plot_format to -TXXX
-            args += " -T%s" %(plot_format)
+            plot_format = "png"
+            args.append("-Tpng")
+            plot['cmd'] = ' '.join(args)
+        if (plot_format == "svg"):
+            plot_format = (app.builder.format in ["latex"]) and "pdf" or plot_format
     elif options.get("plot_format", None):
         #User definition is higher priority.
         plot_format = options["plot_format"]
